# You can use this arguments in code by its name casted to python var style
# '--docker-commands' ==> 'docker_commands'
#
# args = parser.parse_args()
# print(args.docker_commands)
ARGUMENTS = {
    '--docker-commands': {
        'short_name': '-d',
        'help': 'Additional docker commands',
    },
    '--config': {
        'short_name': '-c',
        'help': '',
    }
}


def parse_default_args(parser):
    parser.add_argument('-c', '--config', help='Custom dct config')
    conf_args, unknown = parser.parse_known_args()
    return conf_args


def parse_args(parser, config):
    subparsers = parser.add_subparsers()
    for name, desc in config['commands'].items():
        parser_append = subparsers.add_parser(
            name,
            help=desc.get('description', 'No description provided'))
        for arg_name, args in ARGUMENTS.items():
            parser_append.add_argument(
                args['short_name'],
                arg_name,
                help=args['help'],
            )
        parser_append.set_defaults(command=name)
    args = parser.parse_args()
    if not getattr(args, 'command', False):
        parser.print_help()
        exit(1)
    return args



import hashlib
import yaml

from docker_command_tool.constants import CONFIG_PATH


def get_config(conf_path=None):
    path = CONFIG_PATH
    if conf_path:
        path = conf_path
    with open(path, 'r') as config_file:
        return yaml.load(config_file)


def hash_func(content):
    md5_hash = hashlib.md5()
    md5_hash.update(''.join(content.split()).encode())
    return md5_hash.hexdigest()


import os
import sys

from docker_command_tool.build import build_container
from docker_command_tool.constants import SUCCESS_CODE


class Command:
    def __init__(self, command):
        self.command = command

    def add_name(self, name):
        self.command += f' {name}'

    def add_list_params(self, params_list):
        if params_list:
            for params in params_list:
                for param in params:
                    self.command += param

    def add_commands(self, commands):
        self.command += f' /bin/bash -c "{commands}"'

    def add_docker_params(self, params):
        if params:
            self.command += f' {params}'


def run_command(command, args, config):
    command_desc = config['commands'][command]
    command_container = command_desc['container']

    if build_container(command_container) != SUCCESS_CODE:
        print('Container build is failed')
        sys.exit(1)

    params_list = get_params_list(command_desc)
    docker_run_command(
        command_container,
        command_desc['cmd'],
        params_list,
        args.docker_commands,
    )


def get_params(param, config):
    params = config.get(param['dsl_name'])
    if not params:
        return

    # This is how we get them from yaml
    params = params.split(' ')
    return [param['docker_command'].format(p) for p in params]


def get_params_list(config):
    PARAMS = (
        {'dsl_name': 'volumes', 'docker_command': ' -v {}'},
        {'dsl_name': 'ports', 'docker_command': ' -p {}'},
        {'dsl_name': 'envs', 'docker_command': ' -e {}'},
        {'dsl_name': 'network', 'docker_command': ' --network {}'},
    )
    params_list = [get_params(p, config) for p in PARAMS]
    return [p for p in params_list if p]


def docker_run_command(
        container_name,
        command,
        params_list,
        docker_params
):
    commands = command.strip('\n').split('\n')
    formatted_commands = ' && '.join(commands)

    docker_command = Command(f'docker run -it --rm')
    docker_command.add_docker_params(docker_params)
    docker_command.add_list_params(params_list)
    docker_command.add_name(container_name)
    docker_command.add_commands(formatted_commands)

    print(docker_command.command)
    os.system(
        docker_command.command
    )






import argparse

from docker_command_tool.commands_docker import run_command
from docker_command_tool.build import load_containers
from docker_command_tool.utils import get_config
from docker_command_tool.cli import parse_args, parse_default_args


def main():
    parser = argparse.ArgumentParser(
        prog='bagga',
        description='BAGGA',
        usage='bagga <command> [<args>]'
    )

    default_args = parse_default_args(parser)

    if default_args.config:
        config = get_config(default_args.config)
    else:
        config = get_config()

    load_containers(config)

    args = parse_args(parser, config)
    command = args.command
    run_command(command, args, config)


if __name__ == '__main__':
    main()



import os
import yaml

from docker_command_tool.utils import hash_func
from docker_command_tool.constants import (
    DOCKERFILE_DIR,
    SUCCESS_CODE,
)


def load_containers(dct_config):
    if not os.path.exists(DOCKERFILE_DIR):
        os.mkdir(DOCKERFILE_DIR)

    containers = dct_config['containers']
    for container_name, container_desc in containers.items():
        dockerfile_path = get_dockerfile_path(container_name)
        with open(dockerfile_path, 'w') as dockerfile:
            dockerfile.write(container_desc)


def get_dockerfile_path(container_name):
    return os.path.join('.', DOCKERFILE_DIR, container_name)


def _rewrite_hash_if_needed(container_name):
    hashes_path = os.path.join('.', DOCKERFILE_DIR, 'hashes.yaml')
    if not os.path.exists(hashes_path):
        open(hashes_path, 'a').close()

    dockerfile_path = get_dockerfile_path(container_name)

    with open(dockerfile_path, 'r') as dockerfile:
        dockerfile_content = dockerfile.read()
        new_hash_value = hash_func(dockerfile_content)

    with open(hashes_path, 'r+') as hashes_store:
        hashes_items = yaml.load(hashes_store) or {}
        current_hash_value = hashes_items.get(container_name)
        if current_hash_value != new_hash_value:
            hashes_items[container_name] = new_hash_value
            hashes_store.truncate(0)
            hashes_store.seek(0)
            hashes_store.write(yaml.dump(hashes_items))
            return True

    return False


def build_container(container_name):
    if _rewrite_hash_if_needed(container_name):
        return os.system(f'docker build -t {container_name} '
                         f'-f {DOCKERFILE_DIR}/{container_name} .')
    return SUCCESS_CODE



from setuptools import setup, find_packages

setup(
    name='bagga',
    packages=find_packages(),
    install_requires=[
        'pyyaml==3.11',
    ],
    version='0.0.6',
    description='Docker command tool',
    author='Vitalii Omelchenko',
    author_email='vitaly.om25@gmail.com',
    url='https://github.com/kai25/docker-command-tool',
    keywords=['Docker', 'cli'],
    entry_points={
            'console_scripts': ['bagga=docker_command_tool:main'],
    },
    classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],

)
